#!/usr/bin/env python
import sys
import h5py
import numpy as np
import psana
import datetime
from pytz import timezone

def create_cxi_skeleton(fname, N):

	# Meta data
	f = h5py.File(fname,'w')
	f.create_dataset('cxi_version',data=140,dtype='i32')
	entry_1 = f.create_group('entry_1')
	entry_1.create_dataset('experiment_identifier',(N,),dtype="S128")
	sample_1 = entry_1.create_group('sample_1')
	sample_1.create_dataset('name',data='Rice Dwarf Virus')
	instrument_1 = entry_1.create_group('instrument_1')
	instrument_1.create_dataset('name',data='LCLS CXI Beamline')
	source_1 = instrument_1.create_group('source_1')
	energy = source_1.create_dataset('energy',shape=(N,), dtype='f32')
	energy.attrs.create('axes',"experiment_identifier")
	energy.attrs.create('units',"J")
	source_1["experiment_identifier"] = h5py.SoftLink('/entry_1/experiment_identifier')
	
	# Front detector CsPad (not assembled)
        detector_1 = instrument_1.create_group('detector_1')
        distance = detector_1.create_dataset('distance', data=0.23)
        distance.attrs.create('units','m')
        data = detector_1.create_dataset('data',shape=(N, 32, 185, 388), dtype='f32')
        data.attrs.create('axes',"experiment_identifier:module_identifier:y:x")
        data_white = detector_1.create_dataset('data_white',shape=(1000, 32, 185, 388), dtype='f32')
        data_white.attrs.create('axes',"experiment_identifier:module_identifier:y:x")
        detector_1.create_dataset('module_identifier',data=[str(x) for x in range(0,32)])
        detector_1["experiment_identifier"] = h5py.SoftLink('/entry_1/experiment_identifier')
        data = detector_1.create_dataset('pixel_x_pos',shape=(32, 185, 388), dtype='f32')
        data.attrs.create('units','m')
        data = detector_1.create_dataset('pixel_y_pos',shape=(32, 185, 388), dtype='f32')
        data.attrs.create('units','m')

	# Back detector CsPad 2x2 (not assembled)
        detector_2 = instrument_1.create_group('detector_2')
        distance = detector_2.create_dataset('distance', data=2.4)
        distance.attrs.create('units','m')
        data = detector_2.create_dataset('data',shape=(N, 2, 388, 185), dtype='f32')
        data.attrs.create('axes',"experiment_identifier:y:x")
        data_white = detector_2.create_dataset('data_white',shape=(1000, 2, 388, 185), dtype='f32')
        data_white.attrs.create('axes',"experiment_identifier:y:x")
        detector_2["experiment_identifier"] = h5py.SoftLink('/entry_1/experiment_identifier')
        data = detector_2.create_dataset('pixel_x_pos',shape=(2, 388, 185), dtype='f32')
        data.attrs.create('units','m')
        data = detector_2.create_dataset('pixel_y_pos',shape=(2, 388, 185), dtype='f32')
        data.attrs.create('units','m')

	# Front detector Cspad (assembled)
        image_1 = entry_1.create_group('image_1')
        image_1["detector_1"] = h5py.SoftLink('/entry_1/instrument_1/detector_1')
        image_1["source_1"] = h5py.SoftLink('/entry_1/instrument_1/source_1')
        data = image_1.create_dataset('data',shape=(N, 1750, 1750), dtype='f32')
        data.attrs.create('axes',"experiment_identifier:y:x")
        data_white = image_1.create_dataset('data_white',shape=(1000, 1750, 1750), dtype='f32')
        data_white.attrs.create('axes',"experiment_identifier:y:x")
        image_1.create_dataset('data_type',data='intensities')
        image_1.create_dataset('data_space',data='diffraction')
        image_1["experiment_identifier"] = h5py.SoftLink('/entry_1/experiment_identifier')

	# Back detector CsPad 2x2 (assembled)
	image_1 = entry_1.create_group('image_2')
        image_1["detector_1"] = h5py.SoftLink('/entry_1/instrument_1/detector_1')
        image_1["source_1"] = h5py.SoftLink('/entry_1/instrument_1/source_1')
        data = image_1.create_dataset('data',shape=(N, 400, 400), dtype='f32')
        data.attrs.create('axes',"experiment_identifier:y:x")
	data_white = image_1.create_dataset('data_white',shape=(1000, 400, 400), dtype='f32')
        data_white.attrs.create('axes',"experiment_identifier:y:x")
        image_1.create_dataset('data_type',data='intensities')
        image_1.create_dataset('data_space',data='diffraction')
        image_1["experiment_identifier"] = h5py.SoftLink('/entry_1/experiment_identifier')
        lcls = entry_1.create_group('LCLS')
        lcls.create_dataset('timestamp',(N,),dtype="i64")
        lcls.create_dataset('fiducial',(N,),dtype="i64")
        
        return f

# Psana configuration
psana.setConfigFile('psana.cfg')

# Read in selected hits (run, timestamp, fiducial)
with h5py.File('selection.h5', 'r') as f:
	run = f['run'][:].astype(np.int32)
	lclstime = f['time'][:]
	fiducials = f['fiducial'][:]

# Total nr. of (selected) events
num_events = len(lclstime)

# Create CXI skeleton
fout = create_cxi_skeleton('cxidb-33.cxi', num_events)

# Keep track of current run nr.
current_run_nr = 0

# For each selected event
for i in range(num_events):

	# Go to next run
	if run[i] != current_run_nr:
		ds = psana.DataSource('exp=cxi86715:run=%d:idx' % run[i])
	        current_run = ds.runs().next()
	
	# Save detector metrology to CXI file
        if(i == 0):
		
		# Front detector
                cls = ds.env().calibStore()
                X = cls.get(psana.ndarray_float64_1, psana.Source('DetInfo(CxiDs2.0:Cspad.0)'), 'x-pix-coords').reshape((32,185,388))
                Y = cls.get(psana.ndarray_float64_1, psana.Source('DetInfo(CxiDs2.0:Cspad.0)'), 'y-pix-coords').reshape((32,185,388))
                fout['/entry_1/instrument_1/detector_1/pixel_x_pos'][:] = X*1.0e-6
                fout['/entry_1/instrument_1/detector_1/pixel_y_pos'][:] = Y*1.0e-6

		# Back detector
                cls = ds.env().calibStore()
                # For some reason the 2x2 come in this weird shape, so I transpose them
                X = cls.get(psana.ndarray_float64_1, psana.Source('DetInfo(CxiDg2.0:Cspad2x2.0)'), 'x-pix-coords').reshape((185, 388, 2)).T
                Y = cls.get(psana.ndarray_float64_1, psana.Source('DetInfo(CxiDg2.0:Cspad2x2.0)'), 'y-pix-coords').reshape((185, 388, 2)).T
                fout['/entry_1/instrument_1/detector_2/pixel_x_pos'][:] = X*1.0e-6
                fout['/entry_1/instrument_1/detector_2/pixel_y_pos'][:] = Y*1.0e-6
                
	# Extract psana event
	time = psana.EventTime(int(lclstime[i]), fiducials[i])
	evt = current_run.event(time)
	if evt is None:
		print 'Could not find event', i, ':', lclstime[i], fiducials[i]
		continue
	
	# Read and save photon energy
        bld = evt.get(psana.Bld.BldDataEBeamV7, psana.Source('BldInfo(EBeam)'))
        photon_energy_ev = bld.ebeamPhotonEnergy()
        fout['/entry_1/instrument_1/source_1/energy'][i] = photon_energy_ev*1.60218e-19

	# Save identifiers (timestamp, fiducial)
        obj = evt.get(psana.EventId)
        timestamp = obj.time()[0]+obj.time()[1]*1e-9
        time = datetime.datetime.fromtimestamp(timestamp, tz=timezone('utc'))
        time = time.astimezone(tz=timezone('US/Pacific'))
        identifier =  "LCLS_%s_r%04u_%s_%x" % (time.strftime("%Y%m%d"),
                                               obj.run(), 
                                               time.strftime("%H%M%S"),
                                               obj.fiducials())
        fout['/entry_1/experiment_identifier'][i] = identifier
        fout['/entry_1/LCLS/fiducial'][i]  = fiducials[i]
        fout['/entry_1/LCLS/timestamp'][i] = lclstime[i]

	# Read in front detector (see psana.cfg for details)
	calib_data = evt.get(psana.ndarray_float64_3,
	                     psana.Source('DetInfo(CxiDs2.0:Cspad.0)'), 'calibrated').astype(np.float32)
	recon_data = evt.get(psana.ndarray_float64_2,
	                     psana.Source('DetInfo(CxiDs2.0:Cspad.0)'), 'reconstructed').astype(np.float32)

	# Read in back detector (see psana.cfg for details)
	calib2x2_data = evt.get(psana.ndarray_float64_3,
	                     psana.Source('DetInfo(CxiDg2.0:Cspad2x2.0)'), 'calibrated2x2').astype(np.float32)
	recon2x2_data = evt.get(psana.ndarray_float64_2,
	                     psana.Source('DetInfo(CxiDg2.0:Cspad2x2.0)'), 'reconstructed2x2').astype(np.float32)
		
	# Save front detector to CXI file
	fout['/entry_1/instrument_1/detector_1/data'][i] = calib_data
        fout['/entry_1/image_1/data'][i] = recon_data
        
	# Save back detector to CXI file
        fout['/entry_1/image_2/data'][i] = recon2x2_data
        fout['/entry_1/instrument_1/detector_2/data'][i] = calib2x2_data.T

        # print status	
	sys.stderr.write("\rProcessed event %d/%d in run %d" % (i+1, num_events, run[i]))


# Add shared masks to CXI file
with h5py.File('mask_cspad2x2.h5', 'r') as f:
	fout['/entry_1/image_2/'].create_dataset('mask_shared', data=np.int32(f['/data/data'][:])) 
with h5py.File('mask_cspad.h5', 'r') as f:
	fout['/entry_1/image_1/'].create_dataset('mask_shared', data=np.int32(f['/data/data'][:]))

# Save first 1000 frames (non-hits, non-darks) from run 97 as estimate for background
ds = psana.DataSource('exp=cxi86715:run=97:xtc')

# Stop after 1000 frames
i = 0
n = 1000
while i < n:

	# Iterate through events
	evt = ds.events().next()

	# Read in front detector (see psana.cfg for details)
	calib_data = evt.get(psana.ndarray_float64_3,
			     psana.Source('DetInfo(CxiDs2.0:Cspad.0)'), 'calibrated').astype(np.float32)
	recon_data = evt.get(psana.ndarray_float64_2,
	                    psana.Source('DetInfo(CxiDs2.0:Cspad.0)'), 'reconstructed').astype(np.float32)

	# Read in back detector (see psana.cfg for details)
	calib2x2_data = evt.get(psana.ndarray_float64_3,
				psana.Source('DetInfo(CxiDg2.0:Cspad2x2.0)'), 'calibrated2x2').astype(np.float32)
	recon2x2_data = evt.get(psana.ndarray_float64_2,
				psana.Source('DetInfo(CxiDg2.0:Cspad2x2.0)'), 'reconstructed2x2').astype(np.float32)

	# Check if event is a dark frame
	if recon2x2_data.sum() < 50000:
		pass

	else:
		# Save front detector to CXI file
		fout['/entry_1/instrument_1/detector_1/data_white'][i] = calib_data
		fout['/entry_1/image_1/data_white'][i] = recon_data
        
		# Save back detector to CXI file
		fout['/entry_1/image_2/data_white'][i] = recon2x2_data
		fout['/entry_1/instrument_1/detector_2/data_white'][i] = calib2x2_data.T
		
		# Increment
		i += 1

# Close CXI file
fout.close()
